@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import math
-from pathlib import Path
 
 import plotly.graph_objects as plotly_go
 from plotly.subplots import make_subplots
@@ -24,6 +23,7 @@ from simmate.calculators.vasp.workflows.relaxation.quality04 import (
 from simmate.calculators.vasp.workflows.static_energy.quality04 import (
     StaticEnergy__Vasp__Quality04,
 )
+from simmate.visualization.plotting import PlotlyFigure
 from simmate.workflow_engine import Workflow
 
 # from simmate.calculators.vasp.database.relaxation import StagedRelaxation
@@ -49,7 +49,6 @@ class Relaxation__Vasp__Staged(Workflow):
     """
 
     description_doc_short = "runs a series of relaxations (00-04 quality)"
-    # database_table = StagedRelaxation
 
     subworkflows = [
         Relaxation__Vasp__Quality00,
@@ -84,37 +83,29 @@ class Relaxation__Vasp__Staged(Workflow):
         for i, current_task in enumerate(cls.subworkflows[1:]):
             preceding_task = cls.subworkflows[i]  # will be one before because of [:1]
             state = current_task.run(
-                structure={
-                    "database_table": preceding_task.database_table.table_name,
-                    "directory": result["directory"],  # uses preceding result
-                    "structure_field": "structure_final",
-                },
+                structure=result,  # this is the result of the last run
                 command=command,
                 directory=directory / current_task.name_full,
             )
             result = state.result()
 
-        # we return the final step but update the directory to the parent one
-        result["directory"] = directory
-        return result
+        # when updating the original entry, we want to use the data from the
+        # final result.
+        final_result = {
+            "structure": result.to_toolkit(),
+            "energy": result.energy,
+            "band_gap": result.band_gap,
+            "is_gap_direct": result.is_gap_direct,
+            "energy_fermi": result.energy_fermi,
+            "conduction_band_minimum": result.conduction_band_minimum,
+            "valence_band_maximum": result.valence_band_maximum,
+        }
+        return final_result
 
     @classmethod
-    def _get_final_energy_series(cls, df, directory: str):
-        """
-        A utility that grabs the final energies from each stage. Useful for
-        grabbing convergence information.
-        """
-        for subflow in cls.subworkflows:
-            pass
-
-    @classmethod
-    def get_series_convergence_plot(cls, **filter_kwargs):
-
+    def get_energy_series(cls, **filter_kwargs):
         directories = (
-            cls.database_table.objects.filter(
-                workflow_name=cls.name_full,
-                **filter_kwargs,
-            )
+            cls.all_results.filter(**filter_kwargs)
             .values_list("directory", flat=True)
             .all()
         )
@@ -135,14 +126,24 @@ class Relaxation__Vasp__Staged(Workflow):
                     energy_series.append(None)
             all_energy_series.append(energy_series)
 
-        # -------
+        return all_energy_series
+
+
+class StagedSeriesConvergence(PlotlyFigure):
+    method_type = "classmethod"
+
+    def get_plot(
+        workflow,  # Relaxation__Vasp__Staged
+        **filter_kwargs,
+    ):
+        all_energy_series = workflow.get_energy_series(**filter_kwargs)
 
         figure = make_subplots(
-            rows=math.ceil((len(cls.subworkflows) - 1) / 3),
+            rows=math.ceil((len(workflow.subworkflows) - 1) / 3),
             cols=3,
         )
 
-        for i in range(len(cls.subworkflows) - 1):
+        for i in range(len(workflow.subworkflows) - 1):
 
             xs = []
             ys = []
@@ -166,27 +167,68 @@ class Relaxation__Vasp__Staged(Workflow):
 
             # Update xaxis properties
             figure.update_xaxes(
-                title_text=cls.subworkflows[i + 1].name_full,
+                title_text=f"{workflow.subworkflows[i + 1].name_full}",
                 row=row,
                 col=col,
             )
             figure.update_yaxes(
-                title_text=cls.subworkflows[i].name_full,
+                title_text=workflow.subworkflows[i].name_full,
                 row=row,
                 col=col,
             )
 
+        figure.update_layout(
+            title="Energy per atom (eV) comparison for each stage",
+            showlegend=False,
+        )
         return figure
 
-    @classmethod
-    def view_series_convergence_plot(cls, **kwargs):
-        figure = cls.get_series_convergence_plot(**kwargs)
-        figure.show(renderer="browser")
 
-    @classmethod
-    def write_series_convergence_plot(cls, directory: Path, **kwargs):
-        figure = cls.get_series_convergence_plot(**kwargs)
-        figure.write_html(
-            directory / "convergence__staged_relaxations.html",
-            include_plotlyjs="cdn",
+class StagedSeriesHistogram(PlotlyFigure):
+    method_type = "classmethod"
+
+    def get_plot(
+        workflow,  # Relaxation__Vasp__Staged
+        **filter_kwargs,
+    ):
+
+        all_energy_series = workflow.get_energy_series(**filter_kwargs)
+
+        figure = plotly_go.Figure()
+
+        for i in range(len(workflow.subworkflows) - 1):
+
+            diffs = []
+            for energy_series in all_energy_series:
+                if energy_series[i + 1] and energy_series[i]:
+                    d = energy_series[i + 1] - energy_series[i]
+                    diffs.append(d)
+
+            scatter = plotly_go.Histogram(
+                x=diffs,
+                name=(
+                    f"{workflow.subworkflows[i].name_full} "
+                    f"--> {workflow.subworkflows[i+1].name_full}"
+                ),
+            )
+            figure.add_trace(trace=scatter)
+
+        figure.update_layout(
+            barmode="overlay",
+            xaxis_title_text="Energy per atom change (eV)",
+            yaxis_title_text="Structures (#)",
+            bargap=0.05,
+            legend=dict(
+                yanchor="top",
+                y=0.99,
+                xanchor="left",
+                x=0.01,
+            ),
         )
+
+        return figure
+
+
+# register all plotting methods to the database table
+for _plot in [StagedSeriesConvergence, StagedSeriesHistogram]:
+    _plot.register_to_class(Relaxation__Vasp__Staged)
