@@ -42,6 +42,12 @@ def list_options(options: list) -> int:
 
     """
 
+    # if only only option is given, we want to save the user time and just move to
+    # the next step
+    if len(options) == 1:
+        print(f"Automatically selected `{options[0]}` as it is the only option.")
+        return 0
+
     for i, item in enumerate(options):
         number = str(i + 1).zfill(2)
         print(f"\t({number}) {item}")
@@ -117,36 +123,38 @@ def explore():
     prefix = "\n\n[bold green]"
 
     from simmate.workflows.utilities import (
-        get_list_of_workflows_by_type,
+        get_all_workflow_types,
+        get_calculators_by_type,
         get_workflow,
-        get_workflow_types,
+        get_workflow_names_by_type,
     )
 
     print(f"{prefix}What type of analysis are you interested in?")
-    workflow_types = get_workflow_types()
+    workflow_types = get_all_workflow_types()
     type_index = list_options(workflow_types)
     selected_type = workflow_types[type_index]
 
-    # TODO: have the user select a calculator for this analysis. For now,
-    # we are assuming VASP because those are our only workflows
+    print(f"{prefix}Which calculator do you want to use?")
+    calculator_names = get_calculators_by_type(selected_type)
+    calc_index = list_options(calculator_names)
+    selected_calculator = calculator_names[calc_index]
 
     print(f"{prefix}What settings preset do you want to see the description for?")
-    presets = get_list_of_workflows_by_type(selected_type, full_name=False)
+    presets = get_workflow_names_by_type(
+        selected_type,
+        calculator_name=selected_calculator,
+        full_name=False,
+    )
     present_index = list_options(presets)
     selected_preset = presets[present_index]
 
-    final_workflow_name = selected_type + ".vasp." + selected_preset
-    # BUG: I assume vasp for now, but this should change when I add workflows
-    # from new calculators.
+    # combine selections into our final workflow name
+    final_workflow_name = f"{selected_type}.{selected_calculator}.{selected_preset}"
 
     print(f"{prefix}===================== {final_workflow_name} =====================")
 
     # now we load this workflow and print the docstring.
-    workflow = get_workflow(
-        workflow_name=final_workflow_name,
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
+    workflow = get_workflow(workflow_name=final_workflow_name)
 
     # extra import in order to render markdown in the terminal
     # from: https://rich.readthedocs.io/en/stable/markdown.html
@@ -160,7 +168,8 @@ def explore():
     console = Console()
 
     print(f"{prefix}Description:\n")
-    description = Markdown(dedent(workflow.__doc__))
+    doc = workflow.description_doc
+    description = Markdown(dedent(doc))
     console.print(description)
 
     print(f"{prefix}Parameters:\n")
@@ -182,10 +191,10 @@ def list_all():
     This lists off all available workflows.
     """
 
-    from simmate.workflows.utilities import get_list_of_all_workflows
+    from simmate.workflows.utilities import get_all_workflow_names
 
     print("These are the workflows that have been registerd:")
-    all_workflows = get_list_of_all_workflows()
+    all_workflows = get_all_workflow_names()
     for i, workflow in enumerate(all_workflows):
         workflow_number = str(i + 1).zfill(2)
         print(f"\t({workflow_number}) {workflow}")  # gives "(01) example-flow"
@@ -202,11 +211,7 @@ def show_config(workflow_name: str):
 
     from simmate.workflows.utilities import get_workflow
 
-    workflow = get_workflow(
-        workflow_name=workflow_name,
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
+    workflow = get_workflow(workflow_name)
 
     workflow.show_config()
 
@@ -226,11 +231,7 @@ def setup_only(context: Context, workflow_name: str):
 
     from simmate.workflows.utilities import get_workflow
 
-    workflow = get_workflow(
-        workflow_name=workflow_name,
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
+    workflow = get_workflow(workflow_name)
     kwargs_input = parse_parameters(context=context)
     kwargs_cleaned = workflow._deserialize_parameters(**kwargs_input)
 
@@ -275,11 +276,7 @@ def run_quick(context: Context, workflow_name: str):
 
     from simmate.workflows.utilities import get_workflow
 
-    workflow = get_workflow(
-        workflow_name=workflow_name,
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
+    workflow = get_workflow(workflow_name)
     kwargs_cleaned = parse_parameters(context=context)
 
     result = workflow.run(**kwargs_cleaned)
@@ -291,57 +288,13 @@ def run_quick(context: Context, workflow_name: str):
 
 @workflows_app.command()
 def run(filename: Path):
-    """Runs a workflow locally where parameters are loaded from a yaml file"""
+    """
+    Runs a workflow locally where parameters are loaded from a yaml or toml file
+    """
 
-    import yaml
+    from simmate.workflow_engine import Workflow
 
-    with filename.open() as file:
-        parameters = yaml.full_load(file)  # this is updated below
-
-    from simmate.workflows.utilities import get_workflow
-
-    workflow = get_workflow(
-        # we pop the workflow name so that it is also removed from the rest of kwargs
-        workflow_name=parameters.pop("workflow_name"),
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
-
-    # -------------------------------------------------------------------------
-
-    # SPECIAL CASE -- Running customized workflows from yaml.
-    # !!! When prefect 2.0 is available, I will be able to use **kwargs as an
-    # input for workflow parameters -- in which case, I can move this functionality
-    # to load_input_and_register.
-
-    # For customized workflows, we need to completely change the format
-    # that we provide the parameters. Customized workflows expect parameters
-    # broken into a dictionary of
-    #   {"workflow_base": ..., "input_parameters":..., "updated_settings": ...}
-    if "workflow_base" in parameters.keys():
-
-        parameters["input_parameters"] = {}
-        parameters["updated_settings"] = {}
-
-        for key, update_values in list(parameters.items()):
-            # Skip the base keys
-            if key in [
-                "workflow_base",
-                "input_parameters",
-                "updated_settings",
-            ]:
-                continue
-            # if there is no prefix, then we have a normal input parameter
-            elif not key.startswith("custom__"):
-                parameters["input_parameters"][key] = parameters.pop(key)
-            # Otherwise remove the prefix and add it to the custom settings.
-            else:
-                key_cleaned = key.removeprefix("custom__")
-                parameters["updated_settings"][key_cleaned] = parameters.pop(key)
-
-    # -------------------------------------------------------------------------
-
-    workflow.run(**parameters).result()
+    Workflow.run_from_file(filename).result()
 
 
 @workflows_app.command()
@@ -350,55 +303,7 @@ def run_cloud(filename: Path):
     Submits a workflow to cloud for remote running where parameters are loaded
     from a yaml file
     """
-    # !!! DEV NOTE: This is a copy/paste of the run() command. Condense these
-    # when refactoring
 
-    import yaml
+    from simmate.workflow_engine import Workflow
 
-    with filename.open() as file:
-        parameters = yaml.full_load(file)  # this is updated below
-
-    from simmate.workflows.utilities import get_workflow
-
-    workflow = get_workflow(
-        # we pop the workflow name so that it is also removed from the rest of kwargs
-        workflow_name=parameters.pop("workflow_name"),
-        precheck_flow_exists=True,
-        print_equivalent_import=True,
-    )
-
-    # -------------------------------------------------------------------------
-
-    # SPECIAL CASE -- Running customized workflows from yaml.
-    # !!! When prefect 2.0 is available, I will be able to use **kwargs as an
-    # input for workflow parameters -- in which case, I can move this functionality
-    # to load_input_and_register.
-
-    # For customized workflows, we need to completely change the format
-    # that we provide the parameters. Customized workflows expect parameters
-    # broken into a dictionary of
-    #   {"workflow_base": ..., "input_parameters":..., "updated_settings": ...}
-    if "workflow_base" in parameters.keys():
-
-        parameters["input_parameters"] = {}
-        parameters["updated_settings"] = {}
-
-        for key, update_values in list(parameters.items()):
-            # Skip the base keys
-            if key in [
-                "workflow_base",
-                "input_parameters",
-                "updated_settings",
-            ]:
-                continue
-            # if there is no prefix, then we have a normal input parameter
-            elif not key.startswith("custom__"):
-                parameters["input_parameters"][key] = parameters.pop(key)
-            # Otherwise remove the prefix and add it to the custom settings.
-            else:
-                key_cleaned = key.removeprefix("custom__")
-                parameters["updated_settings"][key_cleaned] = parameters.pop(key)
-
-    # -------------------------------------------------------------------------
-
-    workflow.run_cloud(**parameters)
+    Workflow.run_cloud_from_file(filename).result()
