@@ -192,18 +192,33 @@ INSTALLED_APPS = [
     #   "debug_toolbar",  # django-debug-toolbar  # for debuging and profile-time info
 ]
 
-# We also check if the user has a "applications.yaml" file. In this file, the
+# We also check if the user has a "apps.yaml" file. In this file, the
 # user can provide extra apps to install for Django. We simply append these
-# to our list above
-APPLICATIONS_YAML = SIMMATE_DIRECTORY / "applications.yaml"
-if APPLICATIONS_YAML.exists():
-    with APPLICATIONS_YAML.open() as file:
-        # load the list of extra apps. Since this is really just one line for
-        # each, it is loaded as a single string separated by a space.
-        extra_apps = yaml.full_load(file).split()
+# to our list above. By default we include apps that are packaged with simmate,
+# such as the VASP workflows app.
+DEFAULT_SIMMATE_APPS = [
+    "simmate.workflows.base_flow_types.apps.BaseWorkflowsConfig",
+    "simmate.calculators.vasp.apps.VaspConfig",
+    "simmate.calculators.bader.apps.BaderConfig",
+    "simmate.toolkit.structure_prediction.evolution.apps.EvolutionarySearchConfig",
+]
+APPLICATIONS_YAML = SIMMATE_DIRECTORY / f"{CONDA_ENV}-apps.yaml"
+# create the file if it doesn't exist yet
+if not APPLICATIONS_YAML.exists():
+    with APPLICATIONS_YAML.open("w") as file:
+        content = yaml.dump(DEFAULT_SIMMATE_APPS)
+        file.write(content)
+
+# load apps that the user wants installed
+with APPLICATIONS_YAML.open() as file:
+    SIMMATE_APPS = yaml.full_load(file)
+    # We only load extra apps if the file isn't empty
+    if SIMMATE_APPS:
         # now add each app to our list above so Django loads it.
-        for app in extra_apps:
+        for app in SIMMATE_APPS:
             INSTALLED_APPS.append(app)
+    else:
+        SIMMATE_APPS = []
 
 # --------------------------------------------------------------------------------------
 
@@ -248,10 +263,7 @@ WSGI_APPLICATION = "simmate.website.core.wsgi.application"
 
 AUTH_PASSWORD_VALIDATORS = [
     {
-        "NAME": (
-            "django.contrib.auth."
-            "password_validation.UserAttributeSimilarityValidator"
-        ),  # formatted in this odd way because of line length limit for Black
+        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
     },
     {
         "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
@@ -308,8 +320,8 @@ STATICFILES_DIRS = [DJANGO_DIRECTORY / "static_files"]
 # For the dynamically-created structure files, we need to include the static
 # directory this to work during local testing. This is NOT allowed in a
 # production server, so we don't include it when DEBUG is set to False.
-if DEBUG:
-    STATICFILES_DIRS += [STATIC_ROOT]
+# if DEBUG:
+#     STATICFILES_DIRS += [STATIC_ROOT]
 
 # This sets the django-crispy formating style
 CRISPY_TEMPLATE_PACK = "bootstrap4"
