@@ -14,23 +14,26 @@ import logging
 import shutil
 import urllib
 import warnings
+from functools import cache
 from pathlib import Path
 
 import pandas
 import yaml
-
-# This line does NOTHING but rename a module. I have this because I want to use
-# "table_column.CharField(...)" instead of "models.CharField(...)" in my Models.
-# This let's beginners read my higher level classes and instantly understand what
-# each thing represents -- without them needing to understand
-# that Django Model == Database Table. Experts may find this annoying, so I'm
-# sorry :(
 from django.db import models  # , transaction
 from django.db import models as table_column
+from django.utils.module_loading import import_string
 from django.utils.timezone import datetime
+from django_filters import rest_framework as django_api_filters
 from django_pandas.io import read_frame
 from rich.progress import track
 
+# The "as table_column" line does NOTHING but rename a module.
+# I have this because I want to use "table_column.CharField(...)" instead
+# of "models.CharField(...)" in my Models. This let's beginners read my
+# higher level classes and instantly understand what each thing represents
+# -- without them needing to understand that Django Model == Database Table.
+# Experts may find this annoying, so I'm sorry :(
+
 
 class SearchResults(models.QuerySet):
     """
@@ -121,8 +124,8 @@ class SearchResults(models.QuerySet):
 
     def to_archive(self, filename: Path | str = None):
         """
-        Writes a compressed zip file using the table's base_info attribute.
-        Underneath, the file is written in a csv format.
+        Writes a compressed zip file using the table's `archive_fieldset`
+        attribute. Underneath, the file is written in a csv format.
 
         This is useful for small making archive files and reloading fixtures
         to a separate database.
@@ -160,22 +163,16 @@ class SearchResults(models.QuerySet):
         # convert to path obj
         filename = Path(filename)
 
-        # grab the base_information, and if ID is not present, add it
-        base_info = self.model.base_info
-        if "id" not in base_info:
-            base_info.append("id")
-        # if "created_at" not in base_info:
-        #     base_info.append("created_at")
-        # if "updated_at" not in base_info:
-        #     base_info.append("updated_at")
+        # grab the list of fields that we want to store
+        fieldset = self.model.archive_fieldset
 
         # We want to load the entire table, but only grab the fields that
-        # are in base_info.
-        base_objs = self.only(*base_info)
+        # we will be storing in the archive.
+        base_objs = self.only(*fieldset)
 
         # now convert these objects to a pandas dataframe, again using just
-        # the base_info columns
-        df = base_objs.to_dataframe(fieldnames=base_info)
+        # the archive columns
+        df = base_objs.to_dataframe(fieldnames=fieldset)
 
         # Write the data to a csv file
         # OPTIMIZE: is there a better format that pandas can write to?
@@ -229,17 +226,6 @@ class DatabaseTable(models.Model):
     class Meta:
         abstract = True
 
-    base_info: list[str] = []
-    """
-    The base information for this database table and only these fields are stored
-    when the `to_archive` method is used. Using the columns in this list, all 
-    other columns for this table can be calculated, so the columns in this list 
-    are effectively the "raw data".
-    """
-    # TODO: setting base_info is becoming boilerplate. Consider making this
-    # into a method like to_toolkit that automatically looks at all mix-ins
-    # and creates this list (i.e. involves _base_info properties)
-
     created_at = table_column.DateTimeField(
         auto_now_add=True,
         blank=True,
@@ -366,6 +352,49 @@ class DatabaseTable(models.Model):
     data from this table.
     """
 
+    archive_fields: list[str] = []
+    """
+    The base information for this database table and only these fields are stored
+    when the `to_archive` method is used. Columns excluded from this list can 
+    be calculated quickly and will therefore not be stored. Therefore, this list
+    can be thought of as the "raw data".
+    
+    The columns from mix-ins will automatically be added. If you would like to
+    remove one of these columns, you can add "--" to the start of the column
+    name (e.g. "--energy" will not store the energy).
+    
+    To see all archive fields, see the `archive_fieldset` property.
+    """
+
+    api_filters: dict = {}
+    """
+    Configuration of fields that can be filtered in the REST API and website 
+    interface. This follows the format used by django-filters.
+    
+    For example...
+    ``` python
+    filter_fields = {
+        column1=["exact"],
+        column2=["range"],
+        column3=BooleanFilter(...),
+    }
+    ```
+    
+    See the `api_filterset` property for the final filter object.
+    """
+
+    exclude_from_summary: list[str] = []
+    """
+    When writing output summaries, these columns will be ignored. This is useful
+    if you have a column for storing raw data that isn't friendly to read in
+    the yaml format. 'structure' is an example of a field we'd want to
+    exclude because its not very readable and is available elsewhere.
+    """
+
+    # -------------------------------------------------------------------------
+    # Core methods accessing key information and writing summary files
+    # -------------------------------------------------------------------------
+
     @classmethod
     @property
     def table_name(cls) -> str:
@@ -379,11 +408,280 @@ class DatabaseTable(models.Model):
         """
         return cls.__name__
 
+    @classmethod
+    def get_column_names(cls) -> list[str]:
+        """
+        Returns a list of all the column names for this table and indicates which
+        columns are related to other tables. This is primarily used to help
+        view what data is available.
+        """
+        return [column.name for column in cls._meta.get_fields()]
+
+    @classmethod
+    def show_columns(cls):
+        """
+        Prints a list of all the column names for this table and indicates which
+        columns are related to other tables. This is primarily used to help users
+        interactively view what data is available.
+        """
+        # Iterate through and grab the columns. Note we don't use get_column_names
+        # here because we are attaching relation data as well.
+        column_names = [
+            column.name + f" (relation to {column.related_model.table_name})"
+            if column.is_relation
+            else column.name
+            for column in cls._meta.get_fields()
+        ]
+
+        # Then use yaml to make the printout pretty (no quotes and separate lines)
+        print(yaml.dump(column_names))
+
+    @classmethod
+    def get_mixins(cls) -> list:  # -> List[DatabaseTable]
+        """
+        Grabs the mix-in Tables that were used to make this class. This will
+        be mix-ins like Structure, Forces, etc. from the
+        `simmate.database.base_data_types` module.
+        """
+        # this must be imported locally because it depends on all other classes
+        # from this module -- and will create circular import issues if outside
+        from simmate.database import base_data_types as simmate_mixins
+
+        return [
+            parent
+            for parent in cls.__bases__
+            if hasattr(simmate_mixins, parent.table_name)
+            and parent.table_name != "DatabaseTable"
+        ]
+
+    @classmethod
+    def get_mixin_names(cls) -> list[str]:
+        """
+        Grabs the mix-in Tables that were used to make this class and returns
+        a list of their names.
+        """
+        return [mixin.table_name for mixin in cls.get_mixins()]
+
+    @classmethod
+    def get_extra_columns(cls) -> list[str]:
+        """
+        Finds all columns that aren't covered by the supported Table mix-ins.
+
+        For example, a table made from...
+
+        ``` python
+        from simmate.database.base_data_types import (
+            table_column,
+            Structure,
+            Forces,
+        )
+
+        class ExampleTable(Structure, Forces):
+            custom_column1 = table_column.FloatField()
+            custom_column2 = table_column.FloatField()
+        ```
+
+        ... would return ...
+
+        ``` python
+        ["custom_column1", "custom_column2"]
+        ```
+        """
+
+        all_columns = cls.get_column_names()
+        columns_w_mixin = [
+            column for mixin in cls.get_mixins() for column in mixin.get_column_names()
+        ]
+        extra_columns = [
+            column
+            for column in all_columns
+            if column not in columns_w_mixin and column != "id"
+        ]
+        return extra_columns
+
+    def write_output_summary(self, directory: Path):
+        """
+        This writes a "simmate_summary.yaml" file with key output information.
+        """
+
+        fields_to_exclude = self.exclude_from_summary + [
+            field for mixin in self.get_mixins() for field in mixin.exclude_from_summary
+        ]
+
+        all_data = {}
+        for column, value in self.__dict__.items():
+            if (
+                value != None
+                and not column.startswith("_")
+                and column not in fields_to_exclude
+            ):
+                all_data[column] = value
+
+        # also add the table name and entry id and website URL
+        all_data["_DATABASE_TABLE_"] = self.table_name
+        all_data["_TABLE_ID_"] = self.id
+
+        # EXPERIMNETAL: adding workflow URL
+        try:
+            from simmate.database.base_data_types import Calculation
+            from simmate.workflows.utilities import get_workflow
+
+            if isinstance(self, Calculation):
+                workflow = get_workflow(self.workflow_name)
+                all_data["_WEBSITE_URL_"] = (
+                    "http://127.0.0.1:8000/workflows/"  # I assume local host for now
+                    f"{workflow.name_type}/{workflow.name_calculator}/"
+                    f"{workflow.name_preset}/{self.id}"
+                )
+        except:
+            pass
+
+        summary_filename = directory / "simmate_summary.yaml"
+        with summary_filename.open("w") as file:
+            content = yaml.dump(all_data)
+            file.write(content)
+
+    def to_dict(self):
+        return {
+            "database_table": self.table_name,
+            "database_id": self.id,
+        }
+
+    @staticmethod
+    def from_dict(source_dict: dict):
+
+        # This method can be return ANY table, so we need to import all of them
+        # here. This is a local import to prevent circular import issues.
+        from simmate.website.workflows import models as all_datatables
+
+        # start by loading the datbase table, which is given as a module path
+        datatable_str = source_dict["database_table"]
+
+        # Import the datatable class -- how this is done depends on if it
+        # is from a simmate supplied class or if the user supplied a full
+        # path to the class
+        # OPTIMIZE: is there a better way to do this?
+        if hasattr(all_datatables, datatable_str):
+            datatable = getattr(all_datatables, datatable_str)
+        else:
+            datatable = import_string(datatable_str)
+
+        # These attributes tells us which structure to grab from our datatable.
+        # The user should have only provided one -- if they gave more, we just
+        # use whichever one comes first.
+        run_id = source_dict.get("run_id")
+        database_id = source_dict.get("database_id")
+        directory = source_dict.get("directory")
+
+        # we must have either a run_id or database_id
+        if not run_id and not database_id and not directory:
+            raise Exception(
+                "You must have either a run_id, database_id, "
+                "or directory provided if you want to load a database entry "
+                "from a dictionary of metadata."
+            )
+
+        # now query the datable with which whichever was provided. Each of these
+        # are unique so all three should return a single calculation.
+        if database_id:
+            database_object = datatable.objects.get(id=database_id)
+        elif run_id:
+            database_object = datatable.objects.get(run_id=run_id)
+        elif directory:
+            database_object = datatable.objects.get(directory=directory)
+
+        return database_object
+
+    # -------------------------------------------------------------------------
+    # Methods for loading results from files
+    # -------------------------------------------------------------------------
+
+    @classmethod
+    def from_directory(cls, directory: Path, as_dict: bool = False):
+        """
+        Loads data from a directory of files
+        """
+
+        # check if we have a VASP directory
+        vasprun_filename = directory / "vasprun.xml"
+        if vasprun_filename.exists():
+            return cls.from_vasp_directory(directory, as_dict=as_dict)
+
+        # TODO: add new elif statements when I begin adding new calculators.
+
+        # If we don't detect any directory, we return an empty dictionary.
+        # We don't print a warning or error for now because users may want
+        # to populate data entirely in python.
+        return {} if as_dict else None
+
+    @classmethod
+    def from_vasp_directory(cls, directory: Path, as_dict: bool = False):
+
+        from simmate.calculators.vasp.outputs import Vasprun
+
+        vasprun = Vasprun.from_directory(directory)
+        return cls.from_vasp_run(vasprun, as_dict=as_dict)
+
+    # -------------------------------------------------------------------------
+    # Methods that handle updating a database entry and its related entries
+    # -------------------------------------------------------------------------
+
+    def update_from_fields(self, **fields_to_update):
+        # go through each key in the dictionary and attach the value to the
+        # attribute of this entry
+        for key, value in fields_to_update.items():
+            setattr(self, key, value)
+
+        # Now we have all data loaded and attached to the database entry, we
+        # can call save() to actually update the database
+        self.save()
+
+    def update_from_toolkit(self, **fields_to_update):
+        """
+        Given fundamental "base info" and toolkit objects, this method will
+        populate all relevant columns.
+
+        Note, base info also corresponds to the `archive_fieldset`, which
+        represents raw data.
+
+        This method is meant for updating existing database entries with new
+        data. If your creating a brand-new database entry, use the
+        `from_toolkit` method instead.
+        """
+        fields_expanded = self.from_toolkit(as_dict=True, **fields_to_update)
+        self.update_from_fields(**fields_expanded)
+
+    def update_from_directory(self, directory: Path):
+        # This is for simple tables. If there are related table entries that
+        # need to be created/updated, then this method should be overwritten.
+        data_from_dir = self.from_directory(directory, as_dict=True)
+        self.update_from_toolkit(**data_from_dir)
+
+    def update_from_results(self, results: dict, directory: Path):
+        """
+        Updates a database from the results of a workflow run.
+
+        Typically this method is not called directly, as it is used within
+        `Workflow._save_to_database` automatically.
+        """
+
+        # First update using the results dictionary
+        self.update_from_toolkit(directory=str(directory), **results)
+
+        # Many calculations and datatables will have a "from_directory" method
+        # that loads data from files. We use this to grab extra fields and
+        # add them to our results.
+        self.update_from_directory(directory)
+
+    # -------------------------------------------------------------------------
+    # Methods that handle updating a database entry and its related entries
+    # -------------------------------------------------------------------------
+
     @classmethod
     def from_toolkit(cls, as_dict: bool = False, **kwargs):
         """
-        Given fundamental "base_info" and toolkit objects, this method will populate
-        all relevant columns.
+        Given fundamental "raw data" and toolkit objects, this method
+        will populate all relevant columns.
 
         If the table is made up of multiple mix-ins, this method iterates through
         each of the `_from_toolkit` methods of those mixins and combines the results.
@@ -421,6 +719,7 @@ class DatabaseTable(models.Model):
         # a database column.
         all_data.pop("structure", None)
         all_data.pop("migration_hop", None)
+        all_data.pop("migration_images", None)
         all_data.pop("band_structure", None)
         all_data.pop("density_of_states", None)
 
@@ -461,83 +760,49 @@ class DatabaseTable(models.Model):
         # return the dictionary
         return all_data if as_dict else cls(**all_data)
 
-    def update_from_toolkit(self, **kwargs):
-        """
-        Given fundamental "base_info" and toolkit objects, this method will populate
-        all relevant columns.
-
-        This method is meant for updating existing database entries with new
-        data. If your creating a brand-new database entry, use the
-        `from_toolkit` method instead.
-        """
-        new_kwargs = self.from_toolkit(as_dict=True, **kwargs)
-        for new_kwarg, new_value in new_kwargs.items():
-            setattr(self, new_kwarg, new_value)
-        self.save()
+    # -------------------------------------------------------------------------
+    # Methods creating new archives
+    # -------------------------------------------------------------------------
 
     @classmethod
-    def _confirm_override(
-        cls,
-        confirm_override: bool,
-        parallel: bool,
-        confirm_sqlite_parallel: bool,
-    ):
+    def to_archive(cls, filename: Path | str = None):
         """
-        A utility to make sure the user wants to load new data into their table
-        and (if they are using sqlite) that they are aware of the risks of
-        parallelizing their loading.
-
-        This utility should not be called directly, as it is used within
-        load_archive and load_remote_archive.
+        Writes the entire database table to an archive file. If you prefer
+        a subset of entries for the archive, use the to_archive method
+        on your SearchResults instead (e.g. MyTable.objects.filter(...).to_archive())
         """
-        # first check if the table has data in it already. We raise errors
-        # to stop the user from doing unneccessary and potentiall destructive
-        # downloads
-        if cls.objects.exists() and not confirm_override:
-            # if the user has a third-party app, we can be more specific with
-            # our error message.
-            if cls._meta.app_label == "third_parties":
-                raise Exception(
-                    "It looks like you're using a third-party database table and "
-                    "that the table already has data in it! This means you already "
-                    "called load_remote_archive and don't need to do it again. "
-                    "If you are trying reload a newer version of this data, make "
-                    "sure you empty this table first. This can be done by "
-                    "reseting your database or manually deleting all objects "
-                    "with `ExampleTable.objects.all().delete()`"
-                )
+        cls.objects.all().to_archive(filename)
 
-            # otherwise warning the user of overwriting data with matching
-            # primary keys -- and ask them to use confirm_override.
-            raise Exception(
-                "It looks like this table already has data in it! By loading an "
-                "archive, you could potentially overwrite this data. The most "
-                "common mistake is non-unique primary keys between your current "
-                "data and the archive -- if there is a duplicate primary key, it "
-                "will overwrite your data. If you are confident the data is safe "
-                "to load into your database, run this command again with "
-                "confirm_override=True."
-            )
+    @classmethod
+    @property
+    def archive_fieldset(cls) -> list[str]:
 
-        # Django and Dask can only handle so much for the parallelization
-        # of database writing with SQLite. So if the user has SQLite as their
-        # backend, we need to stop them from using this feature.
-        from simmate.configuration.django.settings import DATABASES
+        all_fields = ["id", "updated_at", "created_at", "source"]
 
-        if parallel and not confirm_sqlite_parallel and "sqlite3" in str(DATABASES):
-            raise Exception(
-                "It looks like you are trying to run things in parallel but are "
-                "using the default database backend (sqlite3), which is not "
-                "always stable for massively parallel methods. You can still "
-                "do this, but this message serves as a word of caution. "
-                "You If you see error messages pop up saying 'database is "
-                "locked', then your database is not stable at the rate you're "
-                "trying to write data. This is a sign that you should either "
-                "(1) switch to a different database backend such as Postgres "
-                "or (2) reduce the parallelization of your tasks. If you are "
-                "comfortable with these warnings and know what you're doing, "
-                "set confirm_sqlite_parallel=True."
-            )
+        # If calling this method on the base class, just return the sole mix-in.
+        if cls == DatabaseTable:
+            return all_fields
+
+        # Otherwise we need to go through the mix-ins and add their fields to
+        # the list
+        all_fields += [
+            field for mixin in cls.get_mixins() for field in mixin.archive_fields
+        ]
+
+        # Sometimes a column will be disabled by adding "--" in front of the
+        # column name. For example, "--band_gap" would exclude storing the band
+        # gap in the archive. We look for any columns that start with this
+        # and then remove them
+        for field in cls.archive_fields:
+            if field.startswith("--"):
+                all_fields.remove(field.removeprefix("--"))
+            else:
+                all_fields.append(field)
+        return all_fields
+
+    # -------------------------------------------------------------------------
+    # Methods that handle loading results from archives
+    # -------------------------------------------------------------------------
 
     # @transaction.atomic  # We can't have an atomic transaction if we use Dask
     @classmethod
@@ -657,8 +922,7 @@ class DatabaseTable(models.Model):
                 structure = ToolkitStructure.from_database_string(structure_str)
                 entry["structure"] = structure
             # OPTIMIZE: is there a better way to do decide which entries need to be
-            # converted to toolkit objects? This code may be better placed in
-            # base_data_type methods (e.g. a `from_base_info` method for each)
+            # converted to toolkit objects?
 
             # BUG: some columns don't properly convert to python objects, but
             # it seems inconsistent when this is done... For now I just manually
@@ -780,65 +1044,233 @@ class DatabaseTable(models.Model):
         logging.info("Done.")
 
     @classmethod
-    def get_column_names(cls) -> list[str]:
+    def _confirm_override(
+        cls,
+        confirm_override: bool,
+        parallel: bool,
+        confirm_sqlite_parallel: bool,
+    ):
         """
-        Returns a list of all the column names for this table and indicates which
-        columns are related to other tables. This is primarily used to help
-        view what data is available.
+        A utility to make sure the user wants to load new data into their table
+        and (if they are using sqlite) that they are aware of the risks of
+        parallelizing their loading.
+
+        This utility should not be called directly, as it is used within
+        load_archive and load_remote_archive.
         """
-        return [column.name for column in cls._meta.get_fields()]
+        # first check if the table has data in it already. We raise errors
+        # to stop the user from doing unneccessary and potentiall destructive
+        # downloads
+        if cls.objects.exists() and not confirm_override:
+            # if the user has a third-party app, we can be more specific with
+            # our error message.
+            if cls._meta.app_label == "third_parties":
+                raise Exception(
+                    "It looks like you're using a third-party database table and "
+                    "that the table already has data in it! This means you already "
+                    "called load_remote_archive and don't need to do it again. "
+                    "If you are trying reload a newer version of this data, make "
+                    "sure you empty this table first. This can be done by "
+                    "reseting your database or manually deleting all objects "
+                    "with `ExampleTable.objects.all().delete()`"
+                )
+
+            # otherwise warning the user of overwriting data with matching
+            # primary keys -- and ask them to use confirm_override.
+            raise Exception(
+                "It looks like this table already has data in it! By loading an "
+                "archive, you could potentially overwrite this data. The most "
+                "common mistake is non-unique primary keys between your current "
+                "data and the archive -- if there is a duplicate primary key, it "
+                "will overwrite your data. If you are confident the data is safe "
+                "to load into your database, run this command again with "
+                "confirm_override=True."
+            )
+
+        # Django and Dask can only handle so much for the parallelization
+        # of database writing with SQLite. So if the user has SQLite as their
+        # backend, we need to stop them from using this feature.
+        from simmate.configuration.django.settings import DATABASES
+
+        if parallel and not confirm_sqlite_parallel and "sqlite3" in str(DATABASES):
+            raise Exception(
+                "It looks like you are trying to run things in parallel but are "
+                "using the default database backend (sqlite3), which is not "
+                "always stable for massively parallel methods. You can still "
+                "do this, but this message serves as a word of caution. "
+                "You If you see error messages pop up saying 'database is "
+                "locked', then your database is not stable at the rate you're "
+                "trying to write data. This is a sign that you should either "
+                "(1) switch to a different database backend such as Postgres "
+                "or (2) reduce the parallelization of your tasks. If you are "
+                "comfortable with these warnings and know what you're doing, "
+                "set confirm_sqlite_parallel=True."
+            )
+
+    # -------------------------------------------------------------------------
+    # Methods that set up the REST API and filters that can be queried with
+    # -------------------------------------------------------------------------
 
     @classmethod
-    def show_columns(cls):
+    @property
+    @cache
+    def api_filterset(cls) -> django_api_filters.FilterSet:
         """
-        Prints a list of all the column names for this table and indicates which
-        columns are related to other tables. This is primarily used to help users
-        interactively view what data is available.
-        """
-        # Iterate through and grab the columns. Note we don't use get_column_names
-        # here because we are attaching relation data as well.
-        column_names = [
-            column.name + f" (relation to {column.related_model.table_name})"
-            if column.is_relation
-            else column.name
-            for column in cls._meta.get_fields()
-        ]
+        Dynamically creates a Django Filter from a Simmate database table.
 
-        # Then use yaml to make the printout pretty (no quotes and separate lines)
-        print(yaml.dump(column_names))
+        For example, this function would take
+        `simmate.database.third_parties.MatprojStructure`
+        and automatically make the following filter:
+
+        ``` python
+        from simmate.website.core_components.filters import (
+            DatabaseTableFilter,
+            Structure,
+            Thermodynamics,
+        )
+
+
+        class MatprojStrucureFilter(
+            DatabaseTableFilter,
+            Structure,
+            Thermodynamics,
+        ):
+            class Meta:
+                model = MatprojStructure  # this is database table
+                fields = {...} # this combines the fields from Structure/Thermo mixins
+
+            # These attributes are set using the declared filters from
+            # the Structure/Thermo mixins
+            declared_filter1 = ...
+            declared_filter1 = ...
+        ```
+        """
+
+        # ---------------------------------------------------------------------
+        # This is the filter mix-in for the base DatabaseTable class, which
+        # ALL other filters will use. We set this up manually.
+        # TODO: Consider moving this outside of this method.
+        class ApiFilter(django_api_filters.FilterSet):
+            class Meta:
+                table = DatabaseTable
+                fields = dict(
+                    # id=["exact"],  --> automatically added...?
+                    created_at=["range"],
+                    updated_at=["range"],
+                )
+
+            def skip_filter(self, queryset, name, value):
+                """
+                For filter fields that use this method, nothing is done to queryset. This
+                is because the filter is typically used within another field. For example,
+                the `include_subsystems` field is not applied to the queryset, but it
+                is used within the `filter_chemical_system` method.
+                """
+                return queryset
+
+        # ---------------------------------------------------------------------
+
+        # If calling this method on the base class, just return the sole mix-in.
+        if cls == DatabaseTable:
+            return ApiFilter
+
+        # First we need to grab the parent mixin classes of the table. For example,
+        # the MatprojStructure uses the database mixins ['Structure', 'Thermodynamics']
+        # while MatprojStaticEnergy uses ["StaticEnergy"].
+        table_mixins = cls.get_mixins()
+
+        # When we make this filter, we want it to inherit from the filters of
+        # all mixins available. We therefore grab all these filters into a list.
+        # We also add the standard ModelForm class from django-filters.
+        filter_mixins = [ApiFilter]
+        filter_mixins += [mixin.api_filterset for mixin in table_mixins]
+
+        # "Declared Filters" are ones normally set as an attribute when creating
+        # a Django filter, whereas normal filters are provided in a meta. For
+        # example...
+        #
+        # class Structure(filters.FilterSet):
+        #     class Meta:
+        #         model = StructureTable
+        #         fields = dict(
+        #             nsites=["range"],  <-------------- field filter
+        #             nelements=["range"],  <----------- field filter
+        #         )
+        #     include_subsystems = filters.BooleanFilter(
+        #         field_name="include_subsystems",
+        #         label="Include chemical subsystems in results?",
+        #         method="skip_filter",
+        #     ) <-------------------------------------- declared filter
+        #     chemical_system = filters.CharFilter(
+        #         method="filter_chemical_system"
+        #     ) <-------------------------------------- declared filter
+        #
+        # I need to separate these out in order to create our class properly
+        field_filters = {}
+        declared_filters = {}
+        filter_methods = {}
+        for field, conditions in cls.api_filters.items():
+            if isinstance(conditions, django_api_filters.Filter):
+                declared_filters.update({field: conditions})
+                # For the declared filters, there is sometimes a method that
+                # needs to be called. This will be attached to the original
+                # mixin as a method of the same name. Here we check if there
+                # is a method and attach it. For an example of this, see the
+                # "filter_chemical_system" method of the Structure class.
+                if conditions.method and conditions.method != "skip_filter":
+                    filter_methods.update(
+                        {conditions.method: getattr(cls, conditions.method)}
+                    )
+            else:
+                field_filters.update({field: conditions})
+
+        # combine the fields of each filter mixin. Note we use .get_fields instead
+        # of .fields to ensure we get a dictionary back
+        field_filters.update(
+            {
+                field: conditions
+                for mixin in filter_mixins
+                for field, conditions in mixin.get_fields().items()
+            }
+        )
+
+        # Also combine all declared filters from each mixin
+        declared_filters.update(
+            {
+                name: filter_obj
+                for mixin in filter_mixins
+                for name, filter_obj in mixin.declared_filters.items()
+            }
+        )
+
+        # The FilterSet class requires that we set extra fields in the Meta class.
+        # We define those here. Note, we accept all fields by default and the
+        # excluded fields are only those defined by the supported form_mixins -
+        # and we skip the first mixin (which is always DatabaseTableFilter)
+        class Meta:
+            model = cls
+            fields = field_filters
+
+        extra_attributes = {"Meta": Meta, **declared_filters, **filter_methods}
+        # BUG: __module__ may need to be set in the future, but we never import
+        # these forms elsewhere, so there's no need to set it now.
+
+        # Now we dynamically create a new form class that we can return.
+        NewClass = type(cls.table_name, tuple(filter_mixins), extra_attributes)
+
+        # Match the filter name to the table name
+        NewClass.filter_name = cls.table_name
+
+        return NewClass
 
     @classmethod
-    def get_mixins(cls) -> list:  # -> List[DatabaseTable]
+    @property
+    @cache
+    def api_filters_extra(cls):
         """
-        Grabs the mix-in Tables that were used to make this class. This will
-        be mix-ins like Structure, Forces, etc. from the
-        `simmate.database.base_data_types` module.
-        """
-        # this must be imported locally because it depends on all other classes
-        # from this module -- and will create circular import issues if outside
-        from simmate.database import base_data_types as simmate_mixins
+        Finds all columns that aren't covered by the supported Form mix-ins.
 
-        return [
-            parent
-            for parent in cls.__bases__
-            if hasattr(simmate_mixins, parent.table_name)
-            and parent.table_name != "DatabaseTable"
-        ]
-
-    @classmethod
-    def get_mixin_names(cls) -> list[str]:
-        """
-        Grabs the mix-in Tables that were used to make this class and returns
-        a list of their names.
-        """
-        return [mixin.table_name for mixin in cls.get_mixins()]
-
-    @classmethod
-    def get_extra_columns(cls) -> list[str]:
-        """
-        Finds all columns that aren't covered by the supported Table mix-ins.
-
-        For example, a table made from...
+        For example, a form made from the database table...
 
         ``` python
         from simmate.database.base_data_types import (
@@ -859,9 +1291,11 @@ class DatabaseTable(models.Model):
         ```
         """
 
-        all_columns = cls.get_column_names()
+        all_columns = cls.api_filterset.base_filters.keys()
         columns_w_mixin = [
-            column for mixin in cls.get_mixins() for column in mixin.get_column_names()
+            column
+            for mixin in cls.get_mixins()
+            for column in mixin.api_filterset.base_filters.keys()
         ]
         extra_columns = [
             column
