@@ -1,9 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from simmate.utilities import get_doc_from_readme
-
-__doc__ = get_doc_from_readme(__file__)
-
 from .aflow import AflowPrototype, AflowStructure
 from .cod import CodStructure
 from .jarvis import JarvisStructure
