@@ -2,11 +2,10 @@
 
 from simmate.database.base_data_types import (
     BandStructureCalc,
-    CustomizedCalculation,
     DensityofStatesCalc,
     DiffusionAnalysis,
+    Dynamics,
     DynamicsIonicStep,
-    DynamicsRun,
     IonicStep,
     MigrationHop,
     MigrationImage,
@@ -16,5 +15,5 @@ from simmate.database.base_data_types import (
 )
 from simmate.toolkit.structure_prediction.evolution.database import (
     FixedCompositionSearch,
-    StructureSource,
+    SteadystateSource,
 )
