@@ -4,55 +4,62 @@ import logging
 import time
 from pathlib import Path
 
-from simmate.database.workflow_results import FixedCompositionSearch as SearchDatatable
+from simmate.database.workflow_results import FixedCompositionSearch
 from simmate.toolkit import Composition
 from simmate.workflow_engine import Workflow
 
 
-class StructurePrediction__Python__FixedComposition(Workflow):
+class StructurePrediction__Toolkit__FixedComposition(Workflow):
+    """
+    Runs an evolutionary search algorithm on a fixed composition with a fixed
+    number of sites. (e.g. Ca2N or Na4Cl4)
+    """
 
-    use_database = False
-    # consider making a calculation table so that the first step of saving
-    # Evolutionary Search is done automatically. It will also ease the
-    # tracking of output files when many fixed compositions are submitted.
+    database_table = FixedCompositionSearch
 
-    @staticmethod
+    @classmethod
     def run_config(
+        cls,
         composition: str | Composition,
         subworkflow_name: str | Workflow = "relaxation.vasp.staged",
         subworkflow_kwargs: dict = {},
         fitness_field: str = "energy_per_atom",
         max_structures: int = None,
         min_structures_exact: int = None,
-        limit_best_survival: int = None,
-        convergence_limit: float = 0.001,
+        best_survival_cutoff: int = None,
+        convergence_cutoff: float = 0.001,
         nfirst_generation: int = 15,
         nsteadystate: int = 40,
-        singleshot_sources: list[str] = [],
-        steadystate_sources: list[tuple[float, str]] = [
-            (0.30, "RandomSymStructure"),
-            (0.30, "from_ase.Heredity"),
-            (0.10, "from_ase.SoftMutation"),
-            (0.05, "from_ase.MirrorMutation"),
-            (0.05, "from_ase.LatticeStrain"),
-            (0.05, "from_ase.RotationalMutation"),
-            (0.05, "from_ase.AtomicPermutation"),
-            (0.05, "from_ase.CoordinatePerturbation"),
-            (0.05, "ExtremeSymmetry"),
+        singleshot_sources: list[str] = [
+            "third_parties",
+            "prototypes",
+            # "third_party_substituition",
         ],
+        steadystate_sources: dict = {
+            "RandomSymStructure": 0.60,
+            "from_ase.Heredity": 0.25,
+            # "from_ase.SoftMutation": 0.10,
+            "from_ase.MirrorMutation": 0.05,
+            # "from_ase.LatticeStrain": 0.05,
+            "from_ase.RotationalMutation": 0.05,
+            "from_ase.AtomicPermutation": 0.05,
+            # "from_ase.CoordinatePerturbation": 0.05,
+            # "ExtremeSymmetry": 0.05,
+        },
         selector_name: str = "TruncatedSelection",
         selector_kwargs: dict = {},
-        validator_name: str = "PartialCrystalNNFingerprint",
-        validator_kwargs: dict = {},
-        tags: list[str] = None,
+        validator_name: str = "PartialRdfFingerprint",
+        validator_kwargs: dict = {
+            "distance_tolerance": 0.001,
+            "cutoff": 10.0,
+            "bin_size": 0.1,
+        },
         sleep_step: int = 60,
         directory: Path = None,
         write_summary_files: bool = True,
+        run_id: str = None,
         **kwargs,
     ):
-        """
-        Sets up the search engine and its settings
-        """
 
         #######################################################################
         # TODO: Check if there is an existing search and grab it if so.
@@ -76,60 +83,44 @@ class StructurePrediction__Python__FixedComposition(Workflow):
         #     if source not in past_singleshot_sources
         # ]
         # BUG: what if I want to rerun any of these even though its been ran before?
-        # An example would be rerunning substituitions when new structures are available
         #######################################################################
 
         logging.info(f"Setting up evolutionary search for {composition}")
 
+        # grab the calculation table linked to this workflow run
+        search_datatable = cls.database_table.objects.get(run_id=run_id)
+
         # Convergence criteria and stop conditions can be set based on the
-        # number of atoms in the composition. Here we try to set reasons criteria
+        # number of atoms in the composition. Here we try to set reasonable criteria
         # for these if a user-input was not given. Note we are using max(..., N)
         # to set an absolute minimum for these.
         n = composition.num_atoms
         min_structures_exact = min_structures_exact or max([int(30 * n), 100])
         max_structures = max_structures or max([int(n * 250 + n**2.75), 1500])
-        limit_best_survival = limit_best_survival or max([int(30 * n + n**2), 100])
-
-        #  Add the search entry to the DB.
-        search_datatable = SearchDatatable(
-            composition=composition.formula,
-            subworkflow_name=subworkflow_name,
-            subworkflow_kwargs=subworkflow_kwargs,
-            fitness_field=fitness_field,
-            max_structures=max_structures,
+        best_survival_cutoff = best_survival_cutoff or max([int(30 * n + n**2), 100])
+        search_datatable.update_from_fields(
             min_structures_exact=min_structures_exact,
-            limit_best_survival=limit_best_survival,
-            convergence_limit=convergence_limit,
-            nfirst_generation=nfirst_generation,
-            nsteadystate=nsteadystate,
-            selector_name=selector_name,
-            selector_kwargs=selector_kwargs,
-            validator_name=validator_name,
-            validator_kwargs=validator_kwargs,
-            tags=tags,
-            sleep_step=sleep_step,
+            max_structures=max_structures,
+            best_survival_cutoff=best_survival_cutoff,
         )
-        search_datatable.save()
 
+        # sometimes the conditions are already met by a previous search so we
+        # check for this up front.
         if search_datatable.check_stop_condition():
             logging.info("Looks like this search was already ran by someone else!")
             return
 
+        logging.info("Finished setup")
+        logging.info(
+            f"Assigned this to FixedCompositionSearch id={search_datatable.id}."
+        )
+
         # See if the singleshot sources have been ran yet. For restarted calculations
         # this will likely not be needed (unless a new source was added). But for
         # new searches/compositions, this will submit all individuals from the
         # single shot sources before we even start the steady-state runs
         search_datatable._check_singleshot_sources(directory)
 
-        # Initialize the steady state sources by saving their config information
-        # to the database.
-        search_datatable._init_steadystate_sources_to_db(steadystate_sources)
-
-        logging.info("Finished setup")
-        logging.info(
-            f"Assigned this to FixedCompositionSearch id={search_datatable.id}."
-        )
-
         # this loop will go until I hit 'break' below
         while True:
 
