@@ -21,8 +21,15 @@ misleading because we know that Ca2N has the spacegroup 166
 
 
 import pandas as pd
-from selenium import webdriver
-from tqdm import tqdm
+from rich.progress import track
+
+try:
+    from selenium import webdriver
+except:
+    raise Exception(
+        "To use this method, Selenium is required. Please install it "
+        "with `conda install -c conda-forge selenium"
+    )
 
 # launch the webbrowser
 driver = webdriver.Chrome(
@@ -30,7 +37,7 @@ driver = webdriver.Chrome(
 )  # /snap/bin/chromium gives errors
 
 table_info = []
-for spacegroup in tqdm(range(1, 231)):
+for spacegroup in track(range(1, 231)):
 
     # load the webpage
     driver.get("https://www.cryst.ehu.es/cryst/get_wp.html")
