@@ -2,16 +2,22 @@
 
 from functools import cache  # cached_property doesnt work with classmethod
 
-from prefect.client import get_client
-from prefect.context import FlowRunContext
-from prefect.flows import Flow
-from prefect.orion.schemas.filters import FlowFilter, FlowRunFilter
-from prefect.packaging import OrionPackager
-from prefect.packaging.serializers import PickleSerializer
-from prefect.states import State
-from prefect.tasks import task  # present only for convience imports elsewhere
+try:
+    from prefect.client import get_client
+    from prefect.context import FlowRunContext
+    from prefect.flows import Flow
+    from prefect.orion.schemas.filters import FlowFilter, FlowRunFilter
+    from prefect.packaging import OrionPackager
+    from prefect.packaging.serializers import PickleSerializer
+    from prefect.states import State
+    from prefect.tasks import task  # present only for convience imports elsewhere
 
-from simmate.utilities import async_to_sync
+    from simmate.utilities.async_wrapper import async_to_sync
+except:
+    raise Exception(
+        "Please install prefect with `conda install -c conda-forge prefect anyio`. "
+        "All prefect features will be broken until this is fixed"
+    )
 
 
 class PrefectWorkflow:
@@ -203,6 +209,9 @@ class PrefectWorkflow:
             "python, so this feature is currently disabled."
         )
         # When this is removed, be sure to re-add the test_workflow_cloud unittest
+        # UPDATE: looks like they re-added it in 2.1.0. You'd use the
+        # build_from_flow method:
+        # https://docs.prefect.io/api-ref/prefect/deployments/
 
         from prefect.deployments import Deployment
 
