@@ -8,6 +8,7 @@ import uuid
 from pathlib import Path
 
 import cloudpickle
+import toml
 import yaml
 
 import simmate
@@ -58,14 +59,16 @@ class Workflow:
     """
     Whether to use Simmate database features or not.
     
-    This includes calling the `_register_calculation` and `_save_to_database`
-    methods attached to this workflow.
+    This includes calling the `_register_calculation` and 
+    `_update_database_with_results` methods attached to this workflow.
     
     `_register_calculation` will save a database entry before the workflow
     starts. This is useful to keep track of workflows that have been
     submitted/started but haven't finished yet.
     
-    `_save_to_database` saves the output of the `workup` method to the database.
+    `_update_database_with_results` saves the output of the `workup` method
+    to the database entry -- and this the same entry that was created by 
+    `_register_calculation`.
     """
 
     _parameter_methods = ["run_config", "_run_full"]
@@ -150,10 +153,30 @@ class Workflow:
             source=source,
             **kwargs,
         )
-        result = cls.run_config(**kwargs_cleaned)
+
+        # Finally run the core part of the workflow. This should return a
+        # dictionary object if we have "use_database=True", but can be
+        # any python object if "use_database=False"
+        results = cls.run_config(**kwargs_cleaned)
+
+        # save the result to the database
         if cls.use_database:
-            result["calculation_id"] = cls._save_to_database(
-                result,
+
+            # make sure the workflow is returning a dictionary that be used
+            # to update the database columns. None is also allowed as it
+            # represents an empty dictionary
+            if not isinstance(results, dict) and results != None:
+                raise Exception(
+                    "When using a database table, your `run_config` method must "
+                    "return a dictionary object. The dictionary is used to "
+                    "update columns in your table entry and is therefore a "
+                    "required format. If you do not want to save to the database "
+                    "(and avoid this message), set `use_database=False`"
+                )
+            logging.info("Saving to database and writing outputs")
+            database_entry = cls._update_database_with_results(
+                results=results if results != None else {},
+                directory=kwargs_cleaned["directory"],
                 run_id=kwargs_cleaned["run_id"],
             )
 
@@ -163,13 +186,16 @@ class Workflow:
             logging.info("Compressing result to a ZIP file.")
             make_archive(kwargs_cleaned["directory"])
 
-        logging.info(f"Completed {cls.name_full}")
-        return result
+        # If we made it this far, we successfully completed the workflow run
+        logging.info(f"Completed '{cls.name_full}'")
+
+        # If we are using the database, then we return the database object.
+        # Otherwise, we want to return the original result from run_config
+        return database_entry if cls.use_database else results
 
     @classmethod
     def run_cloud(
         cls,
-        return_state: bool = True,
         tags: list[str] = None,
         **kwargs,
     ):
@@ -191,24 +217,21 @@ class Workflow:
 
         logging.info(f"Submitting new run of `{cls.name_full}` to cloud")
 
-        # To help with tracking the flow in cloud, we create the flow_id up front.
-        kwargs["run_id"] = kwargs.get("run_id", None) or cls._get_run_id()
-        run_id = kwargs["run_id"]  # just for easy reference below
-
+        # To help with tracking the flow in cloud, we load all of the inputs up
+        # front. This will include creating a run_id for us.
+        #
         # If we are submitting using a filename, we don't want to
         # submit to a cluster and have the job fail because it doesn't have
-        # access to the file. We therefore deserialize right before
-        # serializing in the next line in order to ensure parameters that
-        # accept file names are submitted with all necessary data.
-        parameters_deserialized = cls._deserialize_parameters(**kwargs)
-        parameters_serialized = cls._serialize_parameters(**parameters_deserialized)
+        # access to the file. We therefore go through the full load_input process
+        kwargs_cleaned = cls._load_input_and_register(
+            setup_directory=False,
+            write_metadata=False,
+            **kwargs,
+        )
 
-        # Because we often want to save some info to our database even before
-        # the calculation starts/finishes, we do that by calling _register_calc
-        # at this higher level. An example is storing the structure and run id.
-        # Thus, we create and register the run_id up front
-        if cls.use_database:
-            cls._register_calculation(**kwargs)
+        # Some backends can't pickle input parameters, so we need to serialize
+        # them before submission to the queue.
+        parameters_serialized = cls._serialize_parameters(**kwargs_cleaned)
 
         state = SimmateExecutor.submit(
             cls._run_full,  # should this be the run method...?
@@ -218,12 +241,7 @@ class Workflow:
 
         logging.info(f"Successfully submitted (workitem_id={state.pk})")
 
-        # If the user wants the future, return that instead of the run_id
-        if return_state:
-            state.run_id = run_id  # attach the run id as an extra
-            return state
-
-        return run_id
+        return state
 
     @classmethod
     def run_config(cls, **kwargs) -> any:
@@ -235,6 +253,69 @@ class Workflow:
             "When creating a custom workflow, make sure you set a run_config method!"
         )
 
+    # -------------------------------------------------------------------------
+    # Methods for running/submitting workflows from input files (e.g. YAML/TOML)
+    # -------------------------------------------------------------------------
+
+    @classmethod
+    def run_from_file(cls, filename: Path) -> DummyState:
+        """
+        Runs a workflow locally where parameters are loaded from a yaml or toml file
+        """
+        workflow, parameters = cls._load_settings_file(filename)
+        state = workflow.run(**parameters)
+        return state
+
+    @classmethod
+    def run_cloud_from_file(cls, filename: Path):
+        """
+        Submits a workflow to cloud for remote running where parameters are loaded
+        from a yaml or toml file
+        """
+        workflow, parameters = cls._load_settings_file(filename)
+        state = workflow.run_cloud(**parameters)
+        return state
+
+    @classmethod
+    def _load_settings_file(cls, filename: Path):  # -> tuple[Workflow, dict]
+
+        filename = Path(filename)
+
+        # Load the settings to a dictionary using whichever format is given
+        with filename.open() as file:
+            if filename.suffix == ".yaml":
+                parameters = yaml.full_load(file)
+            elif filename.suffix == ".toml":
+                parameters = toml.load(file)
+            # elif filename.suffix == "json":  TODO consider allowing json
+            #     parameters = json.load(file)
+            else:
+                raise Exception(
+                    "Unknown input file format provided. Please make sure your "
+                    "filename ends in either `.yaml` or `.toml`"
+                )
+
+        # If a workflow name is given, then we are actually calling this method
+        # from the base workflow class -- and need to grab the proper workflow
+        # object.
+        if "workflow_name" in parameters.keys():
+
+            # we are loading an external workflow, which will depend on this base
+            # class. We need a local import to prevent circular-import bug.
+            from simmate.workflows.utilities import get_workflow
+
+            workflow = get_workflow(
+                # we pop the workflow name so that it is also removed from the
+                # rest of kwargs
+                workflow_name=parameters.pop("workflow_name"),
+            )
+            return workflow, parameters
+
+        # Otherwise we should be calling the proper workflow subclass already
+        # and just return the workflow class itself.
+        else:
+            return cls, parameters
+
     # -------------------------------------------------------------------------
     # Methods that interact with the Executor class in order to see what
     # has been submitted to cloud.
@@ -262,11 +343,11 @@ class Workflow:
         """
         The database table where calculation information (such as the run_id)
         is stored. The table should use `simmate.database.base_data_types.Calculation`
-
-        In many cases, this table will contain all of the results you need. However,
-        pay special attention to NestedWorkflows, where your results are often tied
-        to a final task.
+        as one of its mix-ins.
         """
+        # OPTIMIZE: a mapping dictionary or some standardized way to name
+        # database tables would simplify this.
+
         flow_type = cls.name_type
         flow_preset = cls.name_preset
 
@@ -292,54 +373,88 @@ class Workflow:
 
             return PopulationAnalysis
         elif flow_type == "dynamics":
-            from simmate.database.base_data_types import DynamicsRun
+            from simmate.database.base_data_types import Dynamics
 
-            return DynamicsRun
+            return Dynamics
         elif flow_type == "diffusion":
-            if "from-images" in flow_preset:
-                from simmate.database.base_data_types import MigrationImage
+            if "from-images" in flow_preset or "single-path" in flow_preset:
+                from simmate.database.base_data_types import MigrationHop
 
-                return MigrationImage
-            else:
+                return MigrationHop
+            elif "all-paths" in flow_preset:
                 from simmate.database.base_data_types import DiffusionAnalysis
 
                 return DiffusionAnalysis
-        elif flow_type == "customized":
-            from simmate.database.base_data_types import CustomizedCalculation
 
-            return CustomizedCalculation
-        else:
-            raise NotImplementedError("Unable to detect proper database table")
+        raise NotImplementedError(
+            "Unable to detect proper database table. Are you sure your workflow "
+            "should be using a table? If not, set `use_database=False` on your "
+            "workflow as shown in the 'basic' example workflow from the guides."
+        )
 
     @classmethod
     @property
     def all_results(cls):  # -> SearchResults
-        # BUG: pdoc raises an error because name_full fails.
-        try:
-            return cls.database_table.objects.filter(workflow_name=cls.name_full).all()
-        except:
-            return
+        """
+        Filters results from the database table down to the results from this
+        workflow (i.e. matching workflow_name)
+        """
+        return cls.database_table.objects.filter(workflow_name=cls.name_full).all()
 
     @classmethod
-    def _save_to_database(cls, result: any, run_id: str):
+    def _update_database_with_results(
+        cls,
+        results: dict,
+        run_id: str,
+        directory: Path,
+    ) -> Calculation:
+        """
+        Take the output of the `run_config` and any extra information and
+        saves it to the database.
 
-        # split our results and corrections (which are given as a dict) into
-        # separate variables
-        vasprun = result["result"]
-        corrections = result["corrections"]
-        directory = result["directory"]
+        An output summary is also written to file for quick viewing.
+        """
 
         # load the calculation entry for this workflow run. This should already
         # exist thanks to the load_input_and_register task.
         calculation = cls.database_table.from_run_context(
             run_id=run_id,
             workflow_name=cls.name_full,
+            workflow_version=cls.version,
         )
 
-        # now update the calculation entry with our results
-        calculation.update_from_vasp_run(vasprun, corrections, directory)
+        # Now update the calculation entry with our results. Typically, all of this
+        # is handled by the calculation table's "update_from" methods, but in
+        # rare cares, we may want to attach an update method directly to the
+        # workflow class. I can only imagine this is used when...
+        #   (1) workflow attributes are important during the update
+        #   (2) when several workflows share a table and need to isolate
+        #       their workup method (e.g. the MigrationHop table for NEB)
+        if hasattr(cls, "update_database_from_results"):
+            # The attribute can also be set to false to disable updates
+            if cls.update_database_from_results:
+                cls.update_database_from_results(
+                    calculation=calculation,
+                    results=results,
+                    directory=directory,
+                )
+        # Otherwise we hand this off to the database object
+        else:
+            calculation.update_from_results(
+                results=results,
+                directory=directory,
+            )
 
-        return calculation.id
+        # write the output summary to file
+        calculation.write_output_summary(directory)
+        # TODO: consider making this optional to improve speedup
+
+        return calculation
+
+    @classmethod
+    def load_completed_calc(cls, directory: Path):
+        # TODO: maybe load the yaml file to get extra kwargs, run_id, etc.
+        return cls.database_table.from_directory(directory)
 
     # -------------------------------------------------------------------------
     # Properties that enforce the naming convention for workflows
@@ -419,7 +534,7 @@ class Workflow:
         is the same as `__doc__`. This attribute is only defined for beginners
         to python and for use in django templates for the website interface.
         """
-        return cls.__doc__
+        return cls.__doc__ or "No description provided"
 
     @classmethod
     @property
@@ -463,7 +578,43 @@ class Workflow:
         Prints a list of all the parameter names for this workflow.
         """
         # use yaml to make the printout pretty (no quotes and separate lines)
-        print(yaml.dump(cls.parameter_names))
+        print("REQUIRED PARAMETERS")
+        print("--------------------")
+        print(yaml.dump(cls.parameter_names_required))
+
+        print("OPTIONAL PARAMETERS (+ their defaults):")
+        print("---------------------------------------")
+        as_list = [{k: v} for k, v in cls.parameter_defaults.items()]
+        print(yaml.dump(as_list))
+        print("*** 'null' indicates the parameter is set with advanced logic\n")
+
+    @classmethod
+    @property
+    def parameter_defaults(cls):
+        """
+        Inspect the run_config and other methods to see what the default
+        value for an input parameter is.
+        """
+        defaults = {}
+        for method in cls._parameter_methods:
+            sig = inspect.signature(getattr(cls, method))
+            for parameter_name in sig.parameters.keys():
+                if parameter_name in cls.parameter_names:
+                    value = sig.parameters[parameter_name]
+                    if value.default == value.empty:
+                        continue  # dont save if the value is none
+                    defaults.update({parameter_name: value.default})
+
+        # For a series of parameters, we want their default values to be loaded
+        # from class attributes if they are not set. To do this, we first check
+        # if the parameter is set in our kwargs dictionary and check if the
+        # value is set to "None". If it is, then we change to the value set as
+        # the class attribute.
+        for parameter_name in cls.parameter_names:
+            if hasattr(cls, parameter_name):
+                defaults[parameter_name] = getattr(cls, parameter_name)
+
+        return defaults
 
     @classmethod
     def get_config(cls):
@@ -495,7 +646,12 @@ class Workflow:
     # -------------------------------------------------------------------------
 
     @classmethod
-    def _load_input_and_register(cls, **parameters: any) -> dict:
+    def _load_input_and_register(
+        cls,
+        setup_directory: bool = True,
+        write_metadata: bool = True,
+        **parameters: any,
+    ) -> dict:
         """
         How the input was submitted as a parameter depends on if we are submitting
         to Prefect Cloud, running the flow locally, or even continuing from a
@@ -539,104 +695,47 @@ class Workflow:
 
         # ---------------------------------------------------------------------
 
-        # STEP 1b: Determine the "primary" input to use for setting the
-        # source (and previous directory)
-        # OPTIMIZE: Is there a better way to do this?
-
-        # Currently I just set a priority of possible parameters that can be
-        # the primary input. I go through each one at a time until I find one
-        # that was provided -- then I exit with that parameter's value.
-        primary_input = None
-        for primary_input_key in [
-            "structure",
-            "migration_hop",
-            "supercell_start",
-        ]:
-            primary_input = parameters.get(primary_input_key, None)
-            primary_input_cleaned = parameters_cleaned.get(primary_input_key, None)
-            if primary_input:
-                break
-
-        # ---------------------------------------------------------------------
-
-        # STEP 2: Load the directory (and copy over from an old directory if necessary)
-
-        # Start by creating a new directory or grabbing the one given. We create
-        # this directory immediately (rather than just passing the name to the
-        # S3Task). We do this because NestedWorkflows often use a parent directory
-        # to organize results.
-        directory = parameters.get("directory", None)
-        directory_cleaned = get_directory(directory)
-
-        # if the user requested, we grab the previous directory as well
-        copy_previous_directory = parameters.get("copy_previous_directory", None)
-        if copy_previous_directory:
-
-            if not primary_input:
-                raise Exception(
-                    "No primary input detected, which is required for copying "
-                    "past directories. This is an experimental feature so "
-                    "please contact our team for more help."
-                )
-
-            # catch incorrect use of this function
-            if not primary_input_cleaned.is_from_past_calc:
-                raise Exception(
-                    "There isn't a previous directory available! Your source "
-                    "structure must point to a past calculation to use this feature."
-                )
-
-            # the past directory should be stored on the input object
-            previous_directory = Path(primary_input_cleaned.database_object.directory)
-
-            # Copy over all files except simmate ones (we have no need for the
-            # summaries or error archives)
-            copy_directory(
-                directory_old=previous_directory,
-                directory_new=directory_cleaned,
-                ignore_simmate_files=True,
-            )
-
-        # SPECIAL CASE for customized flows
-        if "workflow_base" not in parameters_cleaned:
-            parameters_cleaned["directory"] = directory_cleaned
-        else:
-            parameters_cleaned["input_parameters"]["directory"] = directory_cleaned
-
-        # ---------------------------------------------------------------------
-
-        # STEP 3: Load the source of the input object
+        # STEP 2: Load the source of the input object
 
         source = parameters.get("source", None)
 
-        # If we were given a input from a previous calculation, the source should
-        # point directory to that same input. Otherwise we are incorrectly trying
-        # to change what the source is.
-        # "primary_input and" is added to the start to ensure cleaned input exists
-        # and therefore prevent an error/bug.
-        if source and primary_input and primary_input_cleaned.is_from_past_calc:
-            # note primary_input here is a dictionary
-            # assert
-            if not source == primary_input:
-                # only warning for now because this is experimental
-                logging.warning(
-                    "Your source does not match the source of your "
-                    "primary input. Sources are an experimental feature, so "
-                    "this will not affect your results. Still, please report "
-                    "this to our team to help with development. \n\n"
-                    f"SOURCE: {source} \n\n"
-                    f"PRIMARY_INPUT: {primary_input} \n\n"
-                )
+        # User-given source takes the top priority
+        if source:
             source_cleaned = source
         # Check if we have a primary input loaded from a past calculation and
         # default to that as the source.
-        elif primary_input and primary_input_cleaned.is_from_past_calc:
-            source_cleaned = primary_input
-        # Otherwise just use the source given
-        elif source:
-            source_cleaned = source
         else:
-            source_cleaned = None
+            # If no source was given, we try to dynamically determine the source
+            # using the "primary" input.
+            # OPTIMIZE: Is there a better way to do this?
+
+            # Currently I just set a priority of possible parameters that can be
+            # the primary input. I go through each one at a time until I find one
+            # that was provided -- then I exit with that parameter's value.
+            primary_input = None
+            for primary_input_key in [
+                "structure",
+                "migration_hop",
+                "supercell_start",
+            ]:
+                if primary_input_key in parameters_cleaned.keys():
+                    # note we grab the deserialized input
+                    primary_input = parameters_cleaned.get(primary_input_key, None)
+                    break
+
+            if (
+                primary_input
+                and hasattr(primary_input, "source")
+                and primary_input.source
+            ):
+                source_cleaned = primary_input.source
+
+            else:
+                source_cleaned = None
+
+        # before setting the source, we also need it to be json-seralized
+        # for the database
+        source_cleaned = cls._serialize_parameters(source=source_cleaned)["source"]
 
         # SPECIAL CASE for customized flows
         if "workflow_base" not in parameters_cleaned:
@@ -646,11 +745,60 @@ class Workflow:
 
         # ---------------------------------------------------------------------
 
+        # STEP 3: Load the directory (and copy over from an old directory if necessary)
+
+        if setup_directory:
+            # Start by creating a new directory or grabbing the one given. We create
+            # this directory immediately (rather than just passing the name to the
+            # S3Task). We do this because NestedWorkflows often use a parent directory
+            # to organize results.
+            directory = parameters.get("directory", None)
+            directory_cleaned = get_directory(directory)
+
+            # if the user requested, we grab the previous directory as well
+            copy_previous_directory = parameters.get("copy_previous_directory", None)
+            if copy_previous_directory:
+
+                # BUG: I should switch this to checking the source input arg
+
+                if not primary_input:
+                    raise Exception(
+                        "No primary input detected, which is required for copying "
+                        "past directories. This is an experimental feature so "
+                        "please contact our team for more help."
+                    )
+
+                # catch incorrect use of this function
+                if not primary_input.database_object:
+                    raise Exception(
+                        "There isn't a previous directory available! Your source "
+                        "structure must point to a past calculation to use this feature."
+                    )
+
+                # the past directory should be stored on the input object
+                previous_directory = Path(primary_input.database_object.directory)
+
+                # Copy over all files except simmate ones (we have no need for the
+                # summaries or error archives)
+                copy_directory(
+                    directory_old=previous_directory,
+                    directory_new=directory_cleaned,
+                    ignore_simmate_files=True,
+                )
+
+            # SPECIAL CASE for customized flows
+            if "workflow_base" not in parameters_cleaned:
+                parameters_cleaned["directory"] = directory_cleaned
+            else:
+                parameters_cleaned["input_parameters"]["directory"] = directory_cleaned
+
+        # ---------------------------------------------------------------------
+
         # STEP 4: Register the calculation so the user can follow along in the UI
         # and also see which structures/runs have been submitted aready.
 
         parameters_cleaned["run_id"] = (
-            parameters_cleaned.get("run_id", None) or cls._get_run_id()
+            parameters_cleaned.get("run_id", None) or cls._get_new_run_id()
         )
 
         if cls.use_database:
@@ -660,44 +808,48 @@ class Workflow:
 
         # STEP 5: Write metadata file for user reference
 
-        # convert back to json format. We convert back rather than use the original
-        # to ensure the input data is all present. For example, we want to store
-        # structure data instead of a filename in the metadata.
-        # SPECIAL CASE: "if ..." used to catch customized workflows
-        parameters_serialized = (
-            cls._serialize_parameters(**parameters_cleaned)
-            if "workflow_base" not in parameters_cleaned
-            else parameters
-        )
-
-        # We want to write a file summarizing the inputs used for this
-        # workflow run. This allows future users to reproduce the results if
-        # desired -- and it also allows us to load old results into a database.
-        input_summary = dict(
-            workflow_name=cls.name_full,
-            **parameters_serialized,
-        )
-
-        # now write the summary to file in the same directory as the calc.
-        # check the directory and see how many other "simmate_metadata*.yaml" files
-        # already exist. Our new file will be based off of this. Simply,
-        # if the biggest number is 04 --> then we will write a file named
-        # simmate_metadata__05.yaml to keep the count going.
-        count = (
-            len(
-                [
-                    f
-                    for f in directory_cleaned.iterdir()
-                    if f.name.startswith("simmate_metadata_")
-                ]
+        if write_metadata:
+            # convert back to json format. We convert back rather than use the original
+            # to ensure the input data is all present. For example, we want to store
+            # structure data instead of a filename in the metadata.
+            # SPECIAL CASE: "if ..." used to catch customized workflows
+            parameters_serialized = (
+                cls._serialize_parameters(**parameters_cleaned)
+                if "workflow_base" not in parameters_cleaned
+                else parameters
             )
-            + 1
-        )
-        count_str = str(count).zfill(2)
-        input_summary_file = directory_cleaned / f"simmate_metadata_{count_str}.yaml"
-        with input_summary_file.open("w") as file:
-            content = yaml.dump(input_summary)
-            file.write(content)
+
+            # We want to write a file summarizing the inputs used for this
+            # workflow run. This allows future users to reproduce the results if
+            # desired -- and it also allows us to load old results into a database.
+            input_summary = dict(
+                _WORKFLOW_NAME_=cls.name_full,
+                _WORKFLOW_VERSION_=cls.version,
+                **parameters_serialized,
+            )
+
+            # now write the summary to file in the same directory as the calc.
+            # check the directory and see how many other "simmate_metadata*.yaml" files
+            # already exist. Our new file will be based off of this. Simply,
+            # if the biggest number is 04 --> then we will write a file named
+            # simmate_metadata__05.yaml to keep the count going.
+            count = (
+                len(
+                    [
+                        f
+                        for f in directory_cleaned.iterdir()
+                        if f.name.startswith("simmate_metadata_")
+                    ]
+                )
+                + 1
+            )
+            count_str = str(count).zfill(2)
+            input_summary_file = (
+                directory_cleaned / f"simmate_metadata_{count_str}.yaml"
+            )
+            with input_summary_file.open("w") as file:
+                content = yaml.dump(input_summary)
+                file.write(content)
 
         # ---------------------------------------------------------------------
 
@@ -706,7 +858,7 @@ class Workflow:
         return parameters_cleaned
 
     @staticmethod
-    def _get_run_id():
+    def _get_new_run_id():
         """
         Generates a random id to use as a workflow run id.
 
@@ -731,14 +883,20 @@ class Workflow:
 
         table_columns = cls.database_table.get_column_names()
 
+        # as an extra, we need to check for relations and add also check for
+        # "_id" added on to the name in case we want to register the new entry
+        # with this relation. An example of this is "diffusion_analysis_id"
+        # which is a related object and column.
+        for field in cls.database_table._meta.get_fields():
+            if field.is_relation:  # and isinstance(ForeignKey)
+                # we append BOTH but only one should be found during registration
+                table_columns.append(field.name)
+                table_columns.append(f"{field.name}_id")
+
         for parameter in cls.parameter_names:
             if parameter in table_columns:
                 parameters_to_register.append(parameter)
 
-        # check special cases where input parameter doesn't match to a column name
-        if "structure_string" in table_columns:
-            parameters_to_register.append("structure")
-
         # put in alphabetical order for consistent results
         parameters_to_register.sort()
 
@@ -756,44 +914,13 @@ class Workflow:
         `run_prefect_cloud` method and `load_input_and_register` task.
         """
 
-        # We first need to grab the database table where we want to register
-        # the calculation run to. We can grab the table from either...
-        #   1. the database_table attribute
-        #   2. flow_context --> flow_name --> flow --> then grab its database_table
-
-        # If this method is being called on the base Workflow class, that
-        # means we are trying to register a calculation from within a flow
-        # context -- where the context has information such as the workflow
-        # we are using (and the database table linked to that workflow).
-        if cls == Workflow:
-            raise Exception("Checking if this method is ever used")
-
-            from prefect.context import FlowRunContext
-
-            run_context = FlowRunContext.get()
-            workflow = run_context.flow.simmate_workflow
-            database_table = workflow.database_table
-
-        # Otherwise we should be using the subclass Workflow that has the
-        # database_table property set.
-        else:
-            workflow = cls  # we have the workflow class already
-            database_table = cls.database_table
-
-        # Registration is only possible if a table is provided. Some
-        # special-case workflows don't store calculation information bc the flow
-        # is just a quick python analysis.
-        if not database_table:
-            logging.warning("No database table found. Skipping registration.")
-            return
-
         # grab the registration kwargs from the parameters provided and then
         # convert them to a python object format for the database method
         register_kwargs = {
-            key: kwargs.get(key, None) for key in workflow._parameters_to_register
+            key: kwargs.get(key, None) for key in cls._parameters_to_register
         }
         register_kwargs_cleaned = cls._deserialize_parameters(
-            add_defaults_from_attr=False, **register_kwargs
+            add_defaults=False, **register_kwargs
         )
 
         # SPECIAL CASE: The exception to the above is with SOURCE, which needs
@@ -808,14 +935,16 @@ class Workflow:
         # back to json before saving to the database.
         if "workflow_base" in register_kwargs_cleaned:
             parameters_serialized = cls._serialize_parameters(**register_kwargs_cleaned)
-            calculation = database_table.from_run_context(
+            calculation = cls.database_table.from_run_context(
                 workflow_name=cls.name_full,
+                workflow_version=cls.version,
                 **parameters_serialized,
             )
         else:
             # load/create the calculation for this workflow run
-            calculation = database_table.from_run_context(
+            calculation = cls.database_table.from_run_context(
                 workflow_name=cls.name_full,
+                workflow_version=cls.version,
                 **register_kwargs_cleaned,
             )
 
@@ -825,6 +954,18 @@ class Workflow:
     # Methods that hanlde serialization and deserialization of input parameters.
     # -------------------------------------------------------------------------
 
+    @classmethod
+    def _update_with_defaults(cls, parameter_dict: dict) -> dict:
+        """
+        updates a dictoinary default input parameters if they were provided
+        """
+        for parameter_name, default_value in cls.parameter_defaults.items():
+            if (
+                parameter_dict.get(parameter_name, None) is None
+                and default_value is not None
+            ):
+                parameter_dict[parameter_name] = default_value
+
     @classmethod
     def _serialize_parameters(cls, **parameters) -> dict:
         """
@@ -861,7 +1002,7 @@ class Workflow:
                 if parameter_key == "directory":
                     # convert Path to str
                     parameter_value = str(parameter_value)
-                elif parameter_key == "source":
+                elif parameter_key == "source" and isinstance(parameter_value, dict):
                     # recursive call to this function
                     parameter_value = cls._serialize_parameters(**parameter_value)
                 elif parameter_key == "composition":
@@ -890,7 +1031,7 @@ class Workflow:
     @classmethod
     def _deserialize_parameters(
         cls,
-        add_defaults_from_attr: bool = True,
+        add_defaults: bool = True,
         **parameters,
     ) -> dict:
         """
@@ -924,57 +1065,36 @@ class Workflow:
             return parameters_cleaned
         #######
 
-        # For a series of parameters, we want their default values to be loaded
-        # from class attributes if they are not set. To do this, we first check
-        # if the parameter is set in our kwargs dictionary and check if the
-        # value is set to "None". If it is, then we change to the value set as
-        # the class attribute.
-        if add_defaults_from_attr:
-            for parameter in cls.parameter_names:
-                if parameters.get(parameter, None) == None and hasattr(cls, parameter):
-                    parameters_cleaned[parameter] = getattr(cls, parameter)
+        # We populate this dictionary using the default input parameters if
+        # they were provided.
+        if add_defaults:
+            cls._update_with_defaults(parameters_cleaned)
 
         # The remaining checks look to intialize input to toolkit objects using
         # the from_dynamic methods.
         # OPTIMIZE: if I have proper typing and parameter itrospection, I could
         # potentially grab the from_dynamic method on the fly -- rather than
         # doing these repeated steps here.
+        parameter_mappings = {
+            "structure": Structure,
+            "composition": Composition,
+            "migration_hop": MigrationHop,
+            "migration_images": MigrationImages,
+            "supercell_start": Structure,
+            "supercell_end": Structure,
+        }
 
-        structure = parameters.get("structure", None)
-        if structure:
-            parameters_cleaned["structure"] = Structure.from_dynamic(structure)
-        else:
-            parameters_cleaned.pop("structure", None)
+        for parameter, target_class in parameter_mappings.items():
+            if parameter in parameters.keys():
+                parameter_orig = parameters.get(parameter, None)
+                parameters_cleaned[parameter] = target_class.from_dynamic(
+                    parameter_orig
+                )
 
-        if "composition" in parameters.keys():
-            migration_hop = Composition.from_dynamic(parameters["composition"])
-            parameters_cleaned["composition"] = migration_hop
-
-        if "structures" in parameters.keys():
-            structure_filenames = parameters["structures"].split(";")
-            parameters_cleaned["structures"] = [
-                Structure.from_dynamic(file) for file in structure_filenames
-            ]
-
-        if "migration_hop" in parameters.keys():
-            migration_hop = MigrationHop.from_dynamic(parameters["migration_hop"])
-            parameters_cleaned["migration_hop"] = migration_hop
-
-        if "migration_images" in parameters.keys():
-            migration_images = MigrationImages.from_dynamic(
-                parameters["migration_images"]
-            )
-            parameters_cleaned["migration_images"] = migration_images
-
-        if "supercell_start" in parameters.keys():
-            parameters_cleaned["supercell_start"] = Structure.from_dynamic(
-                parameters["supercell_start"]
-            )
-
-        if "supercell_end" in parameters.keys():
-            parameters_cleaned["supercell_end"] = Structure.from_dynamic(
-                parameters["supercell_end"]
-            )
+        # directory and source are two extra parameters that cant be used in the
+        # mapping above because they don't have a `from_dynamic` method.
+        # Note these also pull from 'parameters_cleaned' as they might have been
+        # populated during registration.
 
         if parameters.get("directory", None):
             parameters_cleaned["directory"] = Path(parameters_cleaned["directory"])
